package db

import (
	"applet/app/admin/md"
	"applet/app/db/model"
	"applet/app/utils/logx"
	"reflect"
	"xorm.io/xorm"
)

type NursingHomeExportRecordsDb struct {
	Db *xorm.Engine `json:"db"`
}

func (nursingHomeExportRecordsDb *NursingHomeExportRecordsDb) Set() { // set方法
	nursingHomeExportRecordsDb.Db = Db
}

func (nursingHomeExportRecordsDb *NursingHomeExportRecordsDb) GetNursingHomeExportRecords(id int) (m *model.NursingHomeExportRecords, err error) {
	m = new(model.NursingHomeExportRecords)
	has, err := nursingHomeExportRecordsDb.Db.Where("id =?", id).Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (nursingHomeExportRecordsDb *NursingHomeExportRecordsDb) GetNursingHomeExportRecordsByMonth(year, month string) (m *model.NursingHomeExportRecords, err error) {
	m = new(model.NursingHomeExportRecords)
	has, err := nursingHomeExportRecordsDb.Db.Where("year =? and month =?", year, month).Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (nursingHomeExportRecordsDb *NursingHomeExportRecordsDb) FindNursingHomeExportRecords() (*[]model.NursingHomeExportRecords, error) {
	var m []model.NursingHomeExportRecords
	if err := nursingHomeExportRecordsDb.Db.Desc("id").Find(&m); err != nil {
		return nil, logx.Error(err)
	}
	return &m, nil
}

func (nursingHomeExportRecordsDb *NursingHomeExportRecordsDb) NursingHomeExportRecordsInsert(m *model.NursingHomeExportRecords) (int, error) {
	_, err := nursingHomeExportRecordsDb.Db.InsertOne(m)
	if err != nil {
		return 0, err
	}
	return m.Id, nil
}

func (nursingHomeExportRecordsDb *NursingHomeExportRecordsDb) NursingHomeExportRecordsInsertBySession(session *xorm.Session, m *model.NursingHomeExportRecords) (int, error) {
	_, err := session.InsertOne(m)
	if err != nil {
		return 0, err
	}
	return m.Id, nil
}

func (nursingHomeExportRecordsDb *NursingHomeExportRecordsDb) NursingHomeExportRecordsDelete(id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return Db.In("id", id).Delete(model.NursingHomeExportRecords{})
	} else {
		return Db.Where("id = ?", id).Delete(model.NursingHomeExportRecords{})
	}
}

func (nursingHomeExportRecordsDb *NursingHomeExportRecordsDb) NursingHomeExportRecordsUpdateBySession(session *xorm.Session, id interface{}, m *model.NursingHomeExportRecords, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = session.Where("id=?", id).Cols(forceColums...).Update(m)
	} else {
		affected, err = session.Where("id=?", id).Update(m)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func (nursingHomeExportRecordsDb *NursingHomeExportRecordsDb) NursingHomeExportRecordsList(req md.NursingHomeExportRecordsListReq) (m []model.NursingHomeExportRecords, total int64, err error) {
	sess := nursingHomeExportRecordsDb.Db.Desc("id")
	if req.Kind != 0 {
		sess.And("kind =?", req.Kind)
	}
	if req.StartDate != "" {
		sess.And("create_at >=?", req.StartDate)
	}
	if req.EndDate != "" {
		sess.And("create_at <=?", req.EndDate)
	}
	total, err = sess.Limit(req.Limit, (req.Page-1)*req.Limit).FindAndCount(&m)
	if err != nil {
		return
	}
	return
}
